<?php

$db_exists = file_exists("daypilot.sqlite");

$db = new PDO('sqlite:daypilot.sqlite');

if (!$db_exists) {
    //create the database
    $db->exec("CREATE TABLE task (
    id               INTEGER  PRIMARY KEY,
    name             TEXT,
    start            DATETIME,
    [end]            DATETIME,
    parent_id        INTEGER,
    milestone        BOOLEAN  DEFAULT (0) NOT NULL,
    ordinal          INTEGER,
    ordinal_priority DATETIME,
    complete         INTEGER  DEFAULT (0) NOT NULL
    );");

    $db->exec("CREATE TABLE link (
    id      INTEGER       PRIMARY KEY AUTOINCREMENT,
    from_id INTEGER       NOT NULL,
    to_id   INTEGER       NOT NULL,
    type    VARCHAR (100) NOT NULL
    );");

    $tasks = array(
        array('name' => 'Group 1',
            'id' => 1,
            'start' => null,
            'end' => null,
            'parent_id' => null,
            'complete' => 0,
            'milestone' => false
        ),
        array('name' => 'Task 1',
            'id' => 2,
            'start' => '2021-09-05T00:00:00',
            'end' => '2021-09-07T00:00:00',
            'parent_id' => 1,
            'complete' => 30,
            'milestone' => false
        ),
        array('name' => 'Task 2',
            'id' => 3,
            'start' => '2021-09-07T00:00:00',
            'end' => '2021-09-09T00:00:00',
            'parent_id' => 1,
            'complete' => 75,
            'milestone' => false
        ),
        array('name' => 'Milestone 1',
            'id' => 4,
            'start' => '2021-09-09T00:00:00',
            'end' => '2021-09-09T00:00:00',
            'parent_id' => 1,
            'complete' => 0,
            'milestone' => true
        ),
        array('name' => 'Group 2',
            'id' => 5,
            'start' => null,
            'end' => null,
            'parent_id' => null,
            'complete' => 0,
            'milestone' => false
        ),
        array('name' => 'Task 3',
            'id' => 6,
            'start' => '2021-09-06T00:00:00',
            'end' => '2021-09-07T00:00:00',
            'parent_id' => 5,
            'complete' => 75,
            'milestone' => false
        ),
    );

    $insert = "INSERT INTO task (name, start, end, complete, parent_id, milestone) VALUES (:name, :start, :end, :complete, :parent, :milestone)";
    $stmt = $db->prepare($insert);

    $stmt->bindParam(':name', $name);
    $stmt->bindParam(':start', $start);
    $stmt->bindParam(':end', $end);
    $stmt->bindParam(':complete', $complete);
    $stmt->bindParam(':parent', $parent);
    $stmt->bindParam(':milestone', $milestone);

    foreach ($tasks as $m) {
        $name = $m['name'];
        $start = $m['start'];
        $end = $m['end'];
        $complete = $m['complete'];
        $parent = $m['parent_id'];
        $milestone = $m['milestone'];
        $stmt->execute();
    }

    $links = array(
        array(
            'from_id' => 2,
            'to_id' => 3,
            'type' => "FinishToStart"
        )
    );

    $insert = "INSERT INTO link (from_id, to_id, type) VALUES (:from_id, :to_id, :type)";
    $stmt = $db->prepare($insert);

    $stmt->bindParam(':from_id', $from);
    $stmt->bindParam(':to_id', $to);
    $stmt->bindParam(':type', $type);

    foreach ($links as $m) {
        $from = $m['from_id'];
        $to = $m['to_id'];
        $type = $m['type'];
        $stmt->execute();
    }

}

date_default_timezone_set("UTC");

function db_get_max_ordinal($parent) {
    global $db;
    $str = "SELECT max(ordinal) FROM [task] WHERE [parent_id] = :parent";
    if ($parent == null) {
        $str = str_replace("= :parent", "is null", $str);
        $stmt = $db->prepare($str);
    }
    else {
        $stmt = $db->prepare($str);
        $stmt->bindParam(":parent", $parent);
    }
    $stmt->execute();
    return $stmt->fetchColumn(0) ?: 0;
}

function db_get_task($id) {
    global $db;

    $str = "SELECT * FROM [task] WHERE [id] = :id";
    $stmt = $db->prepare($str);
    $stmt->bindParam(":id", $id);
    $stmt->execute();
    return $stmt->fetch();
}

function db_update_task_parent($id, $parent, $ordinal) {
    global $db;

    $now = (new DateTime("now"))->format('Y-m-d H:i:s');

    $str = "UPDATE [task] SET [parent_id] = :parent, [ordinal] = :ordinal, [ordinal_priority] = :priority WHERE [id] = :id";
    $stmt = $db->prepare($str);
    $stmt->bindParam(":id", $id);
    $stmt->bindParam(":parent", $parent);
    $stmt->bindParam(":ordinal", $ordinal);
    $stmt->bindParam(":priority", $now);
    $stmt->execute();
}

function db_compact_ordinals($parent) {
    $children = db_get_tasks($parent);
    $size = count($children);
    for ($i = 0; $i < $size; $i++) {
        $row = $children[$i];
        db_update_task_ordinal($row["id"], $i);
    }
}

function db_update_task_ordinal($id, $ordinal) {
    global $db;

    $now = (new DateTime("now"))->format('Y-m-d H:i:s');

    $str = "UPDATE [task] SET [ordinal] = :ordinal, [ordinal_priority] = :priority WHERE [id] = :id";
    $stmt = $db->prepare($str);
    $stmt->bindParam(":id", $id);
    $stmt->bindParam(":ordinal", $ordinal);
    $stmt->bindParam(":priority", $now);
    $stmt->execute();
}

function db_update_task($id, $start, $end) {
    global $db;

    $str = "UPDATE [task] SET [start] = :start, [end] = :end WHERE [id] = :id";
    $stmt = $db->prepare($str);
    $stmt->bindParam(":id", $id);
    $stmt->bindParam(":start", $start);
    $stmt->bindParam(":end", $end);
    $stmt->execute();
}

function db_update_task_full($id, $start, $end, $name, $complete, $milestone) {
    global $db;

    $str = "UPDATE [task] SET [start] = :start, [end] = :end, [name] = :name, [complete] = :complete, [milestone] = :milestone WHERE [id] = :id";
    $stmt = $db->prepare($str);
    $stmt->bindParam(":id", $id);
    $stmt->bindParam(":start", $start);
    $stmt->bindParam(":end", $end);
    $stmt->bindParam(":name", $name);
    $stmt->bindParam(":complete", $complete);
    $stmt->bindParam(":milestone", $milestone);
    $stmt->execute();
}

function db_get_tasks($parent) {
    global $db;

    $str = 'SELECT * FROM task WHERE parent_id = :parent ORDER BY ordinal, ordinal_priority desc';
    if ($parent == null) {
        $str = str_replace("= :parent", "is null", $str);
        $stmt = $db->prepare($str);
    }
    else {
        $stmt = $db->prepare($str);
        $stmt->bindParam(':parent', $parent);
    }

    $stmt->execute();
    return $stmt->fetchAll();
}

?>
