import {Injectable} from '@angular/core';
import {Observable} from 'rxjs';
import {HttpClient} from "@angular/common/http";

@Injectable()
export class DataService {

  constructor(private http : HttpClient){
  }

  getTasks(): Observable<any[]> {
    return this.http.get("/api/backend_tasks.php") as Observable<any[]>;
  }

  getLinks(): Observable<any[]> {
    return this.http.get("/api/backend_links.php") as Observable<any[]>;
  }

  moveTask(data: MoveTaskParams): Observable<any> {
    return this.http.post("/api/backend_move.php", data) as Observable<any>;
  }

  moveRow(data: MoveRowParams): Observable<any> {
    return this.http.post("/api/backend_row_move.php", data) as Observable<any>;
  }

  createRow(data: CreateRowParams): Observable<any> {
    return this.http.post("/api/backend_create.php", data) as Observable<any>;
  }

  deleteTask(data: DeleteTaskParams): Observable<any> {
    return this.http.post("/api/backend_task_delete.php", data) as Observable<any>;
  }

  createLink(data: CreateLinkParams): Observable<any> {
    return this.http.post("/api/backend_link_create.php", data) as Observable<any>;
  }

}

export interface MoveTaskParams {
  id: string | number;
  start: string;
  end: string;
}

export interface MoveRowParams {
  source: string | number;
  target: string | number;
  position: "before" | "after" | "child";
}

export interface CreateRowParams {
  start: string;
  end: string;
  name: string;
}

export interface DeleteTaskParams {
  id: string | number;
}

export interface CreateLinkParams {
  from: string | number;
  to: string | number;
  type: string;
}
